# AVR Assembly Language Decoder
# Roie R. Black
# Date: March 8, 2019
###############################
import sys
import json

OPDATA = "ATtiny85.json"

class Decoder:

    def __init__(self,fn):
        self.filename = fn
        if fn.endswith(".hex"):
            self.hex_loader()
        elif fn.endswith(".lst"):
            self.lst_loader()
        self.json_loader()

    def hex_loader(self):
        try:
            fin = open(self.filename)
        except:
            print("Cannot open file:",self.filename)
            sys.exit()
        lines = fin.readlines()
        for l in lines:
            if not l.startswith(":"): continue
            print(l)

    def lst_loader(self):
        try:
            fin = open(self.filename)
        except:
            print("Cannot open listing file:", self.filename)
            sys.exit()
        lines = fin.readlines()
        for l in lines:
            parts = l.split()
            print(parts)

    def json_loader(self):
        try: 
            fin = open(OPDATA)
        except: 
            print("Cannot load json data file:",OPDATA)
            sys.exit()
        jdata = json.load(fin)
        opcodes = {}
        for j in jdata:
            mnem = j["Mnemonic"]
            opcodes[mnem] = j
        self.opcodes = opcodes

    def print_opcodes(self):
        for o in sorted(opcodes.keys()):
            print(o)
            for d in sorted(opcodes[o].keys()):
                print ('\t',"%s:\t" % d,opcodes[o][d])

    def gen_decode(self):
        encodings = {}
        for op in self.opcodes:
            bits = self.opcodes[op]["Bits"]
            nibs = bits.split()
            nibs.append(op)
            if not nibs[0] in encodings:
                encodings[nibs[0]] = []
            encodings[nibs[0]].append(nibs)
        for en in sorted(encodings.keys()):
            print(en)
            level2 = encodings[en]
            for l in level2:
                print('\t',l[1:])

if __name__ == "__main__":
    try:
        fn = sys.argv[1]
    except:
        print("Usage: Python Decoder.py name['.hex','.lst']")
        sys.exit()
    decoder = Decoder(fn)
    decoder.gen_decode()


